import { d } from './dice';

class TableService {

}

/*
  From the DM Guide Book
  Example table p.116 in DM Guide
  d12 + d8 | Encounter
  2 | Sunken ship covered in barnacles (25 percent chance that the ship contains treasure; roll randomly on the treasure tables in chapter 7)
  3 | Sunken ship with reef sharks
  4 | Bed of giant oysters
  5 | Underwater steam vent (25 percent chance that the vent is a portal to the Elemental Plane of Fire)
  ...
  20 | Sunken treasure chest (25 percent chance that it contains something of value; roll treasure randomly using the tables in chapter 7)

  A good index of the tables: https://slyflourish.com/random_tables_of_the_dmg.html

  Other good tables
  https://www.dicegeeks.com/category/random-rpg-tables/
*/

class RandomTable {
  public name: string;
  private dice: number[];
  private indexedResults: TableResult[] = [];

  constructor(tableConfig: TableConfig) {
    this.name = tableConfig.name;
    this.dice = tableConfig.dice;
    this.constructIndex(tableConfig.possibleResults);
  }

  public roll(input?: number) {
    let testRoll: number = this.getRoll();
    if (!this.isValidInput(testRoll)) {
      console.log(testRoll);
      debugger;
    }
    if (!input || !this.isValidInput(input)) {
      input = this.getRoll();
    }
    return this.indexedResults[input - 1];
  }

  private getRoll(): number {
    let answer: number = 0;
    for (let i in this.dice) {
      answer = answer + d(this.dice[i]);
    }
    return answer;
  }

  private isValidInput(input: number) {
    return typeof this.indexedResults[input - 1] !== 'undefined';
  }

  private constructIndex(tableResults: TableResult[]) {
    for (let resultIndex in tableResults) {
      let keys: number[] = tableResults[resultIndex].keys;
      for (let r = Math.min(...keys); r <= Math.max(...keys); r++) {
        this.indexedResults.push(tableResults[resultIndex]);
      }
    }
  }
}

interface TableResult {
  keys: number[];
  value: string;
  nextTable?: string;
  description?: string;
}

interface TableConfig {
  name: string;
  notes?: string;
  tags?: string[];
  dice: number[];
  possibleResults: TableResult[];
}

const tableService: TableService = new TableService();

export {
  RandomTable,
  TableResult,
  TableConfig,
  tableService
}
